<?php

namespace Bixo\Contact\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Bixo\Contact\Http\Requests\ContactsRequest;
use Bixo\Contact\Interfaces\ContactsRepositoryInterface;
use Bixo\Contact\Models\Contacts;
use Bixo\Contact\Forms\Contacts as Form;

/**
 * APIController  class for contacts.
 */
class ContactsAPIController extends BaseController
{

    /**
     * Initialize contacts resource controller.
     *
     * @param type ContactsRepositoryInterface $contacts
     *
     * @return null
     */
    public function __construct(ContactsRepositoryInterface $contacts)
    {
        parent::__construct();
        $this->repository = $contacts;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Contact\Repositories\Criteria\ContactsResourceCriteria::class);
    }

    /**
     * Display a list of contacts.
     *
     * @return Response
     */
    public function index(ContactsRequest $request)
    {
        return $this->repository
            ->setPresenter(\Bixo\Contact\Repositories\Presenter\ContactsPresenter::class)
            ->paginate();
    }

    /**
     * Display contacts.
     *
     * @param Request $request
     * @param Model   $contacts
     *
     * @return Response
     */
    public function show(ContactsRequest $request, Contacts $contacts)
    {
        return $contacts->setPresenter(\Bixo\Contact\Repositories\Presenter\ContactsListPresenter::class);
        ;
    }

    /**
     * Create new contacts.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ContactsRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('contact::contacts.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('contact/contacts/' . $contacts->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('contact/contacts');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the contacts.
     *
     * @param Request $request
     * @param Model   $contacts
     *
     * @return Response
     */
    public function update(ContactsRequest $request, Contacts $contacts)
    {
        try {
            $data = $request->all();

            $contacts->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('contact::contacts.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('contact/contacts/' . $contacts->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('contact/contacts/' . $contacts->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the contacts.
     *
     * @param Model   $contacts
     *
     * @return Response
     */
    public function destroy(ContactsRequest $request, Contacts $contacts)
    {
        try {
            $contacts->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('contact::contacts.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('contact/contacts/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('contact/contacts/' . $contacts->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
